import argparse
import random
import shutil
import sys
import time
import os
from pathlib import Path

def randomize(shanda_path, data_path):
    sub_paths = [x for x in data_path.iterdir() if x.is_dir()]

    for sub_path in sub_paths:
        deck_path = random.choice([x for x in sub_path.iterdir() if x.is_file()])
        out_path = shanda_path / f'{sub_path.parts[-1]}.dck'
        
        try:
            shutil.copyfile(deck_path, shanda_path / "temp.dck")
            os.replace(shanda_path / "temp.dck", out_path)
            print(f'Copied {deck_path} to {out_path}.')
        except:
            sys.stderr.write(f'Error copying {deck_path} to {out_path}.')
    
    print('\n')

def turbo_randomize(shanda_path, data_path):
    sub_paths = [x for x in data_path.iterdir() if x.is_dir()]
    out_paths = [sub_path.parts[-1] for sub_path in sub_paths]
    random.shuffle(sub_paths)
    random.shuffle(out_paths)

    for sub_path, out_file in zip(sub_paths, out_paths):
        deck_path = random.choice([x for x in sub_path.iterdir() if x.is_file()])
        out_path = shanda_path / f'{out_file}.dck'
        
        try:
            shutil.copyfile(deck_path, shanda_path / "temp.dck")
            os.replace(shanda_path / "temp.dck", out_path)
            print(f'Copied {deck_path} to {out_path}.')
        except:
            sys.stderr.write(f'Error copying {deck_path} to {out_path}.')

    print('\n')

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Runs the Shandalar randomizer.')
    parser.add_argument('path', type=Path, help='The directory of the Shandalar decks.')
    parser.add_argument('-t', '--turbo', dest='randomizer', action='store_const',
                    const=turbo_randomize, default=randomize,
                    help='Randomize all decks (default: randomize per enemy type)')
    parser.add_argument('-o', '--once', action='store_true',
                    help='Randomize only once (default: randomize every 30s)')
    args = parser.parse_args()

    data_path = args.path / 'data'

    if not data_path.exists():
        raise RuntimeError('shandalar data folder does not exist')

    print(f'Randomizing decks from {data_path} to {args.path}...')

    if (args.once):
        args.randomizer(args.path, data_path)
    else:
        while True:
            args.randomizer(args.path, data_path)
            time.sleep(30)